# (C) 2004-2006 Veit Wahlich <cru |at| zodia |dot| de>
# Released under the terms of the General Public License.
# Part of the Perl Script Pages. See POD for more information.

package PSP::Config;
our $VERSION='0.7';

=head1 NAME

PSP::Config - the configuration object class for the Perl Script Pages (PSP)
interface, engine and supporting modules.

=cut

my $default={
#	BaseDirectory => './psp-root',
	BaseDirectory => ($ENV{DOCUMENT_ROOT})?($ENV{DOCUMENT_ROOT}):(''),
#	LibDirectory => [$ENV{DOCUMENT_ROOT}.'../cgi-bin/psp',$ENV{DOCUMENT_ROOT}.'../lib'],
	LibDirectory => [],
	FilenameCharacters => 'a-zA-Z0-9./',
	IndexFile => 'index.psp',
	RequireSecureDirectory => 1,
	OpenTag => '<%',
	CloseTag => '%>',
	PrintSecureOpenTag => '<%=',
	PrintOpenTag => '<%==',
#	BaseURI => $ENV{SCRIPT_NAME} || ''
	BaseURI => ''
};


=head1 SYNOPSIS

	use PSP::Config;
	my $conf = new PSP::Config(
			file => $filename,
			hash => {
					key1 => $value1,
					key2 => $value2
				}
		);
	
	my $value = $conf->get($key);
	
	my $value = $conf->{$key};
	
	my $success = $conf->add($key,$value);
	
	my $success = $conf->addHash(\%hash);
	
	my $success = $conf->addHash(
			{
				key1      => 'value',
				key_array => ['value2', 'value3']
			}
		);
	
	my $success = $conf->addFile($filename);


=head1 DESCRIPTION

This modules provides default configuration values, methods for setting and
accessing configuration data and loading configuration from files.
This module was created as a companion to the Perl Script Pages Engine.
See L<PSP::Engine> for more information.


=head1 OVERVIEW


=head2 Object Constructors

=over

=item new

Creates a new PSP::Config object with default values.
Optional a file to load configuration from or a configuration hash may be
passed to the constructor.
The new constructor is the only object constructor for this module.
See L</"new object constructor"> for details

=back


=head2 Object Methods

=over

=item get

Receive a configuration value for a given key from the configuration data.
See L</"get method"> for details

=item add

Adds a single configuration key/value set to the configuration data.
See L</"add method"> for details

=item addHash

Adds a referenced hash or an anonymous hash to the configuration data.
See L</"addHash method"> for details

=item addFile

Loads data from a file and adds it to the object's configuration data.
See L</"addFile method"> for details

=back


=head1 SYNTAX


=head2 new Object Constructor

Creates a new PSP::Config object with default values.
Optional a file to load configuration from or a configuration hash may be
passed to the constructor.
For file format definitions see L</"addFile method">.
For hash format definitions see L</"addHash method">.

	use PSP::Config;
	my $conf = new PSP::Config(
			file => $filename,
			hash => \%hash
		);
	
	use PSP::Config;
	my $conf = new PSP::Config(
			hash => {
					key1 => $value1,
					key2 => $value2
				}
		);

=head3 Required Parameters

None.

=head3 Optional Parameters

The following parameters are key/value hash parameters.

=over

=item file

Filename to load configuration data from.
For file format definitions see L</"addFile method">.

=item hash

A reference to a hash or an anonymous hash with configuration data.
For hash format definitions see L</"addHash method">.

=back

=cut
sub new{
	my $class=shift;
	my %hash=@_ || ();
	# Clone defaults:
	my $self=\%{$default};
	# Load configuration file if assigned:
	if(exists($hash{file})){
		addFile($self,$hash{file});
	}
	# Add configuration from hash if assigned:
	if(exists($hash{hash})){
		addHash($self,$hash{hash});
	}
	bless($self,$class);
	return($self);
}


=head2 get Method

Receive a configuration value for a given key from the configuration data.

	my $value = $conf->get($key);

You may also access the configuration data directly via the object.

	my $value = $conf->{$key};

=head3 Required Parameters

=over

=item $key

Configuration key (directive) to receive configuration value for.

=back

=head3 Optional Parameters

None.

=head3 Return Values

=over

=item $value

Returns the value stored for the given configuration key or undef if not found.

=back

=cut
sub get{
	my $self=shift || return(undef);
	my $key=shift || return(undef);
	if(exists($self->{$key})){
		return($self->{$key});
	}
	else{
		return(undef);
	}
}


=head2 add Method

Adds a single configuration key/value set to the configuration data.

	my $success = $conf->add($key,$value);

If $key describes a simple scalar configuration directive, $value replaces
the default.
If $key is an array configuration directive, $value is appended to the
corresponding values array.

$value may contain special C<@@string@@> macros that are replaced by
environment valiables, for example C<@@DOCUMENT_ROOT@@> will be replaced by the
content of the C<$DOCUMENT_ROOT> environment variable, i.e. "/var/www/html/".

=head3 Required Parameters

=over

=item $key

Configuration key used to store and identify the new value.

=item $value

Configuration data to store for $key.

=back

=head3 Optional Parameters

None.

=head3 Return Values

=over

=item $success

Returns 1 on success or undef on failure.

=back

=cut
sub add{
	my $self=shift || return(undef);
	my $key=shift || return(undef);
	my $value=shift;
	# FIXME: Abort here with "Unknown configuration directive" if no default exists?
	if(exists($self->{$key}) && ref($self->{$key}) eq 'ARRAY'){
		push(@{$self->{$key}},$value);
	}
	else{
		$self->{$key}=$value;
	}
	return(1);
}


=head2 addHash Method

Adds a referenced hash or an anonymous hash to the configuration data.

	my $success = $conf->addHash($hashref);

	my %hash = (
			key1      => 'value',
			key_array => ['value2', 'value3']
		);
	my $success = $conf->addHash(\%hash);
	
	my $success = $conf->addHash(
			{
				key1      => 'value',
				key_array => ['value2', 'value3']
			}
		);

The hash elements can either be standard "key => $value" parameters that call
the L</"add method"> once or "key => [$value1, $value2]" ordered array
parameters that call the L</"add method"> for every value in the array but
always for the same key.

Values may contain environment variable macros. See L</"add method"> for details.

=head3 Required Parameters

=over

=item $hashref

A reference to a hash containing configuration data.

=back

=head3 Optional Parameters

None.

=head3 Return Values

=over

=item $success

Returns 1 on success or undef on failure.

=back

=cut
sub addHash{
	my $self=shift || return(undef);
	my $hash=shift || return(undef);
	my $key;
	foreach $key (keys(%{$hash})){
		if(ref($hash->{$key}) eq 'ARRAY'){
			foreach(@{$hash->{$key}}){
				add($self,$key,$_);
			}
		}
		else{
			add($self,$key,$hash->{$key});
		}
	}
	return(1);
}


=head2 addFile Method

Loads data from a file and adds it to the object's configuration data.

	my $success = $conf->addFile($filename);

Configuration files are ordinary text files and may contain lines of the format
C<ThisIsTheKey=This is a value>, terminated by \n or \r\n.
The system interpretes everything left of the first "=" character as the key,
everything on the right as configuration value, so both key and value parts may
contain i.e. spaces, value even may contain the "=" character without quoting
or escaping.
Empty lines and lines beginning with "#" (comments) are ignored.

Values may contain environment variable macros. See L</"add method"> for details.

=head3 Required Parameters

=over

=item $filename

Name of the file to load configuration data from.

=back

=head3 Optional Parameters

None.

=head3 Return Values

=over

=item $success

Returns 1 on success or undef on failure.

=back

=cut
sub addFile{
	my $self=shift || return(undef);
	my $file=shift || return(undef);
	my $in;
	my($key,$val);
	open($in,'<',$file)
		|| do{
			print(STDERR "Unable to load config file '".$file."': ".$!."\n");
			return(undef);
		};
	while(readline($in)){
		unless('' || /^#/){
			s/\@\@(.*?)\@\@/$ENV{$1}/g;
			s/[\r\n]//g;
			add($self,split(/=/,$_,2));
		}
	}
	close($in);
	return(1);
}

1;

=head1 AUTHOR

Veit Wahlich

E-Mail: cru |at| zodia |dot| de

WWW: L<http://home.ircnet.de/cru/>


=head1 VERSION

v0.7
Wednesday, 18 January 2006


=head1 COPYRIGHT/LICENSE

Copyright 2004-2006 Veit Wahlich

This software is distributed as free (libre) software under the terms of the
GNU General Public License, version 2 E<lt>L<http://www.gnu.org/copyleft/gpl.html>E<gt>.
The author disclaims responsibility of any damage or harm caused directly or
indirectly by usage of this software. Use only at your own risk.
